from sqlmodel import Session, select

from src.config.database import engine


def get_by_column(model, attribute, attribute_value, quantity="single"):
    with Session(engine) as session:
        column = getattr(model, attribute)

        query = select(model).where(column == attribute_value)

        result = session.exec(query)

        if quantity == "multiple":
            return result.all()

        else:
            return result.first()



from fastapi import FastAPI

from src.config.database import create_db_and_tables
from src.public_routes import public_routes

app = FastAPI()


@app.on_event("startup")
def on_startup():
    create_db_and_tables()


@app.get("/")
def docs():
    return {"docs": "see /docs", "redoc": "see /redoc"}


for public_route in public_routes:
    app.include_router(public_route)



from decouple import config
from sqlmodel import SQLModel, create_engine

from src.models.items import Items  # noqa: F401
from src.models.users import Users  # noqa: F401

database_url = config("DATABASE_URL")

engine = create_engine(database_url, echo=True)


def create_db_and_tables():
    SQLModel.metadata.create_all(engine)



from fastapi import APIRouter, Depends

from src.controllers.items import handle_get_items
from src.security.verify_jwt_token import verify_jwt_token

items_routes = APIRouter(dependencies=[Depends(verify_jwt_token)])


@items_routes.get("/items")
def get_items():
    return handle_get_items()



from sqlmodel import Session

from src.config.database import engine


def post(row):
    with Session(engine) as session:
        session.add(row)

        session.commit()

        session.refresh(row)

        return row



from fastapi import APIRouter

from src.controllers.auth import (
    handle_post_auth_login_users,
    handle_post_auth_register_users,
)
from src.models.users import Users

auth_routes = APIRouter()


@auth_routes.post("/auth/register-users")
def post_auth_register_users(user: Users):
    return handle_post_auth_register_users(user)


@auth_routes.post("/auth/login-users")
def post_auth_login_users(user: Users):
    return handle_post_auth_login_users(user)



from sqlmodel import Session, select

from src.config.database import engine


def get_all(model):
    with Session(engine) as session:
        rows = session.exec(select(model)).all()

        return rows



from decouple import config
from fastapi import HTTPException, Request, status
from jose import JWTError, jwt


def verify_jwt_token(request: Request):
    auth_header = request.headers.get("Authorization")

    if not auth_header or not auth_header.startswith("Bearer "):
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Token de autenticação ausente ou inválido",
            headers={"WWW-Authenticate": "Bearer"},
        )

    token = auth_header.split(" ")[1]

    try:
        payload = jwt.decode(
            token, config("SECRET_KEY"), algorithms=[config("ALGORITHM")]
        )

        sub = payload.get("sub")

        email = payload.get("email")

        if sub is None or email is None:
            raise HTTPException(
                status_code=status.HTTP_401_UNAUTHORIZED,
                detail="Token JWT malformado: claims obrigatórias ausentes",
                headers={"WWW-Authenticate": "Bearer"},
            )

        return payload

    except JWTError as e:  # noqa: F841
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Token JWT inválido ou expirado",
            headers={"WWW-Authenticate": "Bearer"},
        )



from sqlalchemy import Column, String, UniqueConstraint
from sqlmodel import Field, SQLModel


class Users(SQLModel, table=True):
    __tablename__ = "users"
    __table_args__ = (UniqueConstraint("email"),)
    id: int | None = Field(default=None, primary_key=True)
    email: str = Field(sa_column=Column(String, unique=True, nullable=False))
    password: str



import re
from datetime import datetime, timedelta

from decouple import config
from jose import jwt
from passlib.hash import sha256_crypt

from src.models.users import Users
from src.repositories.get_by_column import get_by_column
from src.repositories.post import post


def handle_post_auth_register_users(user: Users):
    password = user.password

    if (
        len(password) < 8
        or not re.search(r"[A-Z]", password)
        or not re.search(r"[a-z]", password)
        or not re.search(r"\d", password)
        or not re.search(r"[!@#$%^&*(),.?\":{}|<>]", password)
    ):
        return {
            "success": False,
            "error": "Senha deve ter no mínimo 8 caracteres, incluindo letras maiúsculas, minúsculas, números e caracteres especiais.",
        }

    user.password = sha256_crypt.hash(password)

    new_user = post(user)

    return {"success": True if new_user else False}


def handle_post_auth_login_users(user: Users):
    db_user = get_by_column(Users, "email", user.email)

    if not db_user:
        return {"success": False, "error": "Usuário não encontrado."}

    if not sha256_crypt.verify(user.password, db_user.password):
        return {"success": False, "error": "Senha incorreta."}

    expire = datetime.utcnow() + timedelta(minutes=60)

    payload = {"sub": str(db_user.id), "email": db_user.email, "exp": expire}

    token = jwt.encode(payload, config("SECRET_KEY"), algorithm=config("ALGORITHM"))

    return {"success": True, "access_token": token, "token_type": "bearer"}
